/*
 * Copyright (c) 1997 Ian W. Dall <ian.dall@dsto.defence.gov.au>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Ian W. Dall.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <assert.h>
#include "main.h"
#include "menue.h"
#include "format_disk.h"
#include "scsi_op.h"

void format_disk(void)
{
  enum {Exit = 1, Doit};
  static bool cmplist = True;
  static unsigned int interleave = 0;
  static unsigned int timeout = 3600;
  static struct menue_entry entry_list[] = {
    VAR_RANGE("Timeout (seconds): ", 0, "%u", "%i", Unsigned, timeout, 10, UINT_MAX),
    VAR_TOGGLE("Reset defect list? ", 0, Bool, cmplist, 0),
    VAR_RANGE("Interleave (0 => target default): ", 0, "%u", "%i", Unsigned, interleave, 0, UINT_MAX),
    ACTION("Start formatting.", 0, Doit, 0),
    [9] = ACTION("Exit", 0, Exit, 0)
  };

  static struct menue analyse_menue = MENUE("Format Disk", entry_list);
  while (1) {
    switch (do_menue(analyse_menue)) {
    case Doit:
      {
	int status;
	scsi_format(0, 0, cmplist, interleave, timeout, &status);
      }
      break;
    case EOF:
    default:
      clearerr(stdin);
      return;
    }
  }


}
