/*
 * Copyright (c) 1997 Ian W. Dall <ian.dall@dsto.defence.gov.au>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Ian W. Dall.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

typedef enum {False = 0, True = 1} bool;

typedef char *string;

enum entry_type {Skip, Space, Action, Data};
enum value_type { Int, Unsigned, Bool, BoolComp, String };

union polyp {			/* Polymorphic pointer */
  int * Int;
  unsigned int * Unsigned;
  bool *Bool;
  bool *BoolComp;
  string *String;
};

union poly {			/* Polymorphic value */
  int Int;
  unsigned int Unsigned;
  bool Bool;
  bool BoolComp;
  string String;
};


typedef union poly poly;  
typedef union polyp polyp;
struct menue_entry;

typedef int (validate_function)(struct menue_entry *entryp);

struct menue_entry {
  char * text;
  enum entry_type entry_type;
  bool *disable;
  union {
    struct {
      char * p_format;
      char * s_format;
      enum value_type type;
      polyp value;
      poly min;
      poly max;
      int malloced_value;
    } data;
    int action_code;
  } u;
  int (*hook)(struct menue_entry *entryp);
};

struct menue {
  char *title;
  int len;
  struct menue_entry *list;
};

#define MENUE(title,entries) {title,sizeof(entries)/sizeof(entries[0]), entries}

#define VAR(text, disable, p_format, s_format,type,var,min,max,hook) \
   {text, Data, disable, {data: {p_format, s_format, type, {type: &var}, \
   {type: min}, {type: max}}}, hook}

#define VAR_RANGE(text, disable, p_format, s_format,type,var,min,max) \
   VAR(text, disable, p_format, s_format,type,var,min,max,0)

#define VAR_VALIDATE(text, disable, p_format, s_format,type,var,valid) \
   VAR(text, disable, p_format, s_format, type, var, 0, 0, valid)

#define VAR_TOGGLE(text, disable,type,var,valid) \
   VAR(text, disable, 0, 0, type, var, 0, 0, valid)

#define VAR_NULL VAR(0, 0, 0, 0, 0, 0, 0, 0, 0)

#define ACTION(text, disable, code, hook) {text, Action, disable, {action_code: code}, hook}

#define SKIP() {"", Skip}

#define SPACE() {"", Space}


int do_menue(struct menue menue);

void error(const char *fmt, ...);

void errorx(const char *fmt, ...);

void errorxx(const char *fmt, ...);

void warning(const char *fmt, ...);

void warningx(const char *fmt, ...);

void message(const char *fmt, ...);

char *prompt(const char *fmt,...);

void prompt_any(const char *fmt,...);

int prompt_y_or_n(const char *fmt,...);

void get_ttysize(void);

void display_status(const char *fmt,...);
